#!/bin/sh
# Build a native compiler using the cross-compiler form build.sh
#

if [ ! "$MUSL_CC_BASE" ]
then
    MUSL_CC_BASE=`dirname "$0"`
fi

# Fail on any command failing, show commands:
set -ex

BINUTILS_CONFFLAGS=
GCC_BOOTSTRAP_CONFFLAGS=
MUSL_CONFFLAGS=
GCC_CONFFLAGS=
WITH_SYSROOT=no
NATIVE_BUILD=yes
. "$MUSL_CC_BASE"/defs.sh

# Switch to the CC prefix for all of this
PREFIX="$CC_PREFIX"

# make the sysroot usr directory
if [ ! -e "$PREFIX"/"$TRIPLE"/usr ]
then
    mkdir -p "$PREFIX"/"$TRIPLE"
    ln -sf . "$PREFIX"/"$TRIPLE"/usr
fi
if [ "$WITH_SYSROOT" = "yes" ]
then
    SYSROOT_FLAGS="--with-sysroot=""$PREFIX"/"$TRIPLE"
fi


# 
if [ -z $GCC_URL ];
then
    fetchextract http://ftp.gnu.org/gnu/gcc/gcc-$GCC_VERSION/ gcc-$GCC_VERSION .tar.bz2
else
    fetchextract "$GCC_URL"
    if [ -e $GCC_EXTRACT_DIR ]; then
	mv $GCC_EXTRACT_DIR gcc-$GCC_VERSION
    fi
fi

[ "$GCC_BUILTIN_PREREQS" = "yes" ] && gccprereqs


# linux headers
fetchextract $LINUX_HEADERS_URL
LINUX_HEADERS_DIR=$(stripfileext $(basename $LINUX_HEADERS_URL))

if [ ! -e $LINUX_HEADERS_DIR/configured ]
then
    (
    cd $LINUX_HEADERS_DIR
    make $LINUX_DEFCONFIG ARCH=$LINUX_ARCH
    touch configured
    )
fi
if [ ! -e $LINUX_HEADERS_DIR/installedheaders ]
then
    (
    cd $LINUX_HEADERS_DIR
    make headers_install ARCH=$LINUX_ARCH INSTALL_HDR_PATH="$CC_PREFIX/$TRIPLE"
    touch installedheaders
    )
fi

# musl in CC prefix
PREFIX="$CC_PREFIX/$TRIPLE"
muslfetchextract
# We set both CROSS_COMPILE and CC because CC in the environment overrides
# and CROSS_COMPILE setting
buildinstall 1 musl-$MUSL_VERSION \
    --enable-debug --enable-optimize CROSS_COMPILE="$TRIPLE-" $MUSL_CONFFLAGS
unset PREFIX
PREFIX="$CC_PREFIX"

# if it didn't build libc.so, disable dynamic linking
if [ ! -e "$CC_PREFIX/$TRIPLE/lib/libc.so" ]
then
    GCC_CONFFLAGS="--disable-shared $GCC_CONFFLAGS"
fi

# gcc 2
buildinstall 1 gcc-$GCC_VERSION --target=$TRIPLE --host=$TRIPLE $SYSROOT_FLAGS \
    --enable-languages=$LANGUAGES --disable-libmudflap \
    --disable-libsanitizer --disable-nls \
    $GCC_MULTILIB_CONFFLAGS \
    $GCC_CONFFLAGS

# binutils
fetchextract "$BINUTILS_URL"
BINUTILS_DIR=$(stripfileext $(basename $BINUTILS_URL))

sed -i -e 's,MAKEINFO="$MISSING makeinfo",MAKEINFO=true,g' \
    $BINUTILS_DIR/configure

buildinstall 1 $BINUTILS_DIR --target=$TRIPLE --host=$TRIPLE --disable-werror $SYSROOT_FLAGS \
    $BINUTILS_CONFFLAGS

# un"fix" headers
rm -rf "$CC_PREFIX/lib/gcc/$TRIPLE"/*/include-fixed/ "$CC_PREFIX/lib/gcc/$TRIPLE"/*/include/stddef.h

# make backwards-named compilers for easier cross-compiling
(
    cd "$CC_PREFIX/bin"
    for tool in $TRIPLE-*
    do
        btool=`echo "$tool" | sed 's/-linux-musl/-musl-linux/'`
        if [ -e $tool -a "$tool" != "$btool" -a ! -e "$btool" ] 
	then
	  ln -s $tool $btool
	fi 
    done
)

exit 0
