#!/bin/sh
# Build tarballs of various musl cross-compilers
#
# Example invocation:
#   build-tarballs.sh /opt/cross/musl crossx86- -1.1 i486 x86_64 x32 arm \
#   armeb armhf microblaze mips mipsel mips-sf mipsel-sf powerpc sh4 sh4eb
# 
# Copyright (C) 2012-2014 Gregor Richards
# 
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
# 
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.

if [ ! "$MUSL_CC_BASE" ]
then
    MUSL_CC_BASE=`dirname "$0"`/..
fi

set -ex


cleanup() {
    for pkg in binutils gcc linux kernel-headers musl gmp mpfr mpc
    do
        for bf in configured build built installed
        do
            rm -rf $pkg-*/$bf*
        done
    done
    unset bf

    for pkg in musl-*/ gmp-*/ mpfr-*/ mpc-*/
    do
        (
        cd $pkg &&
        make distclean || true
        )
    done
    unset pkg
}

if [ -e config.sh ]
then
    echo 'Please use a fresh directory.'
    exit 1
fi
touch extraconfig.sh

if [ ! "$4" ]
then
    echo 'Use: .../build-tarballs.sh <install prefix> <tarball prefix> <tarball suffix> <native arch> [other archs]'
    exit 1
fi

PREFIX_BASE="$1"
shift
T_PRE="$1"
shift
T_SUFF="$1"
shift
NATIVE_ARCH="$1"

GNATMAKE="gnatmake"
export GNATMAKE

for ARCH in "$@"
do
    case "$ARCH" in
        armhf)
            TRIPLE="arm-linux-musleabihf"
            ;;
        arm*)
            TRIPLE="$ARCH-linux-musleabi"
            ;;
        x32)
            TRIPLE="x86_64-x32-linux-musl"
            ;;
        *)
            TRIPLE="$ARCH-linux-musl"
            ;;
    esac

    if [ "$ARCH" = "$NATIVE_ARCH" ]
    then
        if [ ! -e "$PREFIX_BASE/bootstrap/done" ]
        then
           (
           mkdir -p build-bootstrap
           cd build-bootstrap	   
	   if [ ! -e extraconfig.sh ]
           then
	      ln -s ../extraconfig.sh .
           fi
            # Make the config.sh
            echo 'ARCH='$ARCH'
TRIPLE='$TRIPLE'
CC_PREFIX="'"$PREFIX_BASE/bootstrap"'"
MAKEOPTS="'"$MAKEOPTS"'"
. ./extraconfig.sh' > config.sh

	    if [ ! -e "$MUSL_CC_BASE"/build.sh ]
            then	
            MUSL_CC_BASE=../"$MUSL_CC_BASE"
	    fi	

            # Build the bootstrap one first
            "$MUSL_CC_BASE"/build.sh
            echo 'ARCH='$ARCH'
TRIPLE='$TRIPLE'
CC_PREFIX="'"$PREFIX_BASE/bootstrap"'"
MAKEOPTS="'"$MAKEOPTS"'"
CC='$TRIPLE'-gcc
export CC
. ./extraconfig.sh' > config.sh
            "$MUSL_CC_BASE"/extra/build-gcc-deps.sh

	    touch $PREFIX_BASE/bootstrap/done
	    )
        fi

        NATIVE_CROSS="$PREFIX_BASE/bootstrap/bin/$TRIPLE"

        # Get rid of dlfcn.h as a cheap hack to disable building plugins
        rm -f "$PREFIX_BASE/bootstrap/$TRIPLE/include/dlfcn.h"
    fi

    if [ ! -e "$PREFIX_BASE/$TRIPLE/done" ]
    then
           (
           mkdir -p build-$TRIPLE
           cd build-$TRIPLE
	   if [ ! -e extraconfig.sh ]
           then
	      ln -s ../extraconfig.sh .
           fi
        # Make the config.sh
        echo 'ARCH='$ARCH'
TRIPLE='$TRIPLE'
CC_PREFIX="'"$PREFIX_BASE/$TRIPLE"'"
MAKEOPTS="'"$MAKEOPTS"'"
CC="'"$NATIVE_CROSS-gcc"' -static -Wl,-Bstatic -static-libgcc"
CXX="'"$NATIVE_CROSS-g++"' -static -Wl,-Bstatic -static-libgcc"
GNATMAKE="'"$NATIVE_CROSS-gnatmake"' -bargs -static -cargs -static -static-libgcc -largs -Wl,-Bstatic -static -static-libgcc -margs"
GNATBIND="'"$NATIVE_CROSS-gnatbind"' -static"
GNATLS="'"$NATIVE_CROSS-gnatls"'"
GNATLINK="'"$NATIVE_CROSS-gnatlink -static -static-libgcc -Wl,-Bstatic"'"
export CC CXX GNATMAKE GNATBIND GNATLS GNATLINK
GCC_BOOTSTRAP_CONFFLAGS=--disable-lto-plugin
GCC_CONFFLAGS=--disable-lto-plugin
. ./extraconfig.sh' > config.sh
            
        # If MUSL_CC_BASE is an absolute directory, then no need for the extra ..
	if [ ! -e $MUSL_CC_BASE/build.sh ]
        then	
         MUSL_CC_BASE=../$MUSL_CC_BASE
	fi	
        # And build
        "$MUSL_CC_BASE"/build.sh
        #sed -E '/^C(C|XX)=/d ; /^export/d' -i config.sh
        "$MUSL_CC_BASE"/extra/build-gprbuild.sh
        echo 'ARCH='$ARCH'
TRIPLE='$TRIPLE'
CC_PREFIX="'"$PREFIX_BASE/$TRIPLE"'"
MAKEOPTS="'"$MAKEOPTS"'"
CC="'"$TRIPLE-gcc"' -static -Wl,-Bstatic -static-libgcc"
CXX="'"$TRIPLE-g++"' -static -Wl,-Bstatic -static-libgcc"
GNATMAKE="'"$NATIVE_CROSS-gnatmake"' -bargs -static -cargs -static -static-libgcc -largs -Wl,-Bstatic -static -static-libgcc -margs"
GNATBIND="'"$NATIVE_CROSS-gnatbind"' -static"
GNATLS="'"$NATIVE_CROSS-gnatls"'"
GNATLINK="'"$NATIVE_CROSS-gnatlink -static -static-libgcc -Wl,-Bstatic"'"
export CC CXX GNATMAKE GNATBIND GNATLS GNATLINK
GCC_BOOTSTRAP_CONFFLAGS=--disable-lto-plugin
GCC_CONFFLAGS=--disable-lto-plugin
. ./extraconfig.sh' > config.sh
        [ "$NO_GCC_DEPS" != "yes" ] && "$MUSL_CC_BASE"/extra/build-gcc-deps.sh

        # Make the tarball
        cp -f extraconfig.sh "$PREFIX_BASE/$TRIPLE/extraconfig.sh"
        (
        cd "$PREFIX_BASE"
        tar -cf - "$TRIPLE/" | gzip -c > "$T_PRE$TRIPLE$T_SUFF.tar.gz"
        )
        touch "$PREFIX_BASE/$TRIPLE/done"
        echo "Cross-compiler prefix built by musl-cross $TRIPLE"
	)
    fi
done

for ARCH in "$@"
do
    case "$ARCH" in
        armhf)
            TRIPLE="arm-linux-musleabihf"
            ;;
        arm*)
            TRIPLE="$ARCH-linux-musleabi"
            ;;
        x32)
            TRIPLE="x86_64-x32-linux-musl"
            ;;
        *)
            TRIPLE="$ARCH-linux-musl"
            ;;
    esac


    if [ ! -e "$PREFIX_BASE/$TRIPLE-native/done" ]
    then
        (
         SYSTEM_CROSS="$PREFIX_BASE/$TRIPLE/bin/$TRIPLE"
           mkdir -p build-$TRIPLE-native
           cd build-$TRIPLE-native
	   if [ ! -e extraconfig.sh ]
           then
	      ln -s ../extraconfig.sh .
           fi

         #rm -Rf gprbuild-gpl-2017-src/
         #rm -Rf xmlada-gpl-2017-src/
         echo 'ARCH='$ARCH'
TRIPLE='$TRIPLE'
CC_PREFIX="'"$PREFIX_BASE/$TRIPLE-native"'"
MAKEOPTS="'"$MAKEOPTS"'"
CC="'"$SYSTEM_CROSS-gcc"' -static -Wl,-Bstatic -static-libgcc"
CC_FOR_BUILD="'"$NATIVE_CROSS-gcc"' -static -Wl,-Bstatic -static-libgcc"
CXX="'"$SYSTEM_CROSS-g++"' -static -Wl,-Bstatic -static-libgcc"
GNATMAKE="'"$SYSTEM_CROSS-gnatmake"' -bargs -static -cargs -static -static-libgcc -largs -Wl,-Bstatic -static -static-libgcc -margs"
GNATMAKE_FOR_BUILD="'"$NATIVE_CROSS-gnatmake"' -bargs -static -cargs -static -static-libgcc -largs -Wl,-Bstatic -static -static-libgcc -margs"
GNATBIND="'"$SYSTEM_CROSS-gnatbind"' -static"
GNATLS="'"$SYSTEM_CROSS-gnatls"'"
GNATLINK="'"$SYSTEM_CROSS-gnatlink -static -static-libgcc -Wl,-Bstatic"'"
export CC CXX GNATMAKE GNATBIND GNATLS GNATLINK
export GNATMAKE_FOR_BUILD CC_FOR_BUILD
GCC_BOOTSTRAP_CONFFLAGS=--disable-lto-plugin
GCC_CONFFLAGS=--disable-lto-plugin
PATH="'"$PREFIX_BASE/$TRIPLE/bin"'":$PATH
PATH="'"$PREFIX_BASE/$TRIPLE/bin/$TRIPLE/bin"'":$PATH
export PATH
. ./extraconfig.sh' > config.sh
        # If MUSL_CC_BASE is an absolute directory, then no need for the extra ..
	if [ ! -e $MUSL_CC_BASE/build.sh ]
        then	
         MUSL_CC_BASE=../$MUSL_CC_BASE
	fi	

        "$MUSL_CC_BASE"/build-native.sh
	NATIVE_BUILD=yes
	export NATIVE_BUILD
        [ "$NO_GCC_DEPS" != "yes" ] && "$MUSL_CC_BASE"/extra/build-gcc-deps.sh
        "$MUSL_CC_BASE"/extra/build-gprbuild.sh
        (
        cd "$PREFIX_BASE"
        tar -cf - "$TRIPLE-native/" | gzip -c > "$T_PRE$TRIPLE-native$T_SUFF.tar.gz"
        )
        touch "$PREFIX_BASE/$TRIPLE-native/done"

        echo "Native compiler prefix built by musl-cross $TRIPLE-native"
	)
    fi
done
